#ifndef HCAL_AMC13_AMC13_FLASH_HH_INCLUDED
#define HCAL_AMC13_AMC13_FLASH_HH_INCLUDED 1

#include <vector>
#include <stdint.h>
#include <iostream>
#include <stdio.h>
#include <string>
#include <fstream>

#include "hcal/amc13/AMC13_utils.hh"
#include "hcal/amc13/AMC13.hh"

namespace cms {
  
  //Class to handle AMC13 flash programming
  class AMC13_flash {
  public:
    //Contructor and Destructor
    AMC13_flash(AMC13_utils*, AMC13*);
    ~AMC13_flash() { };
    
    //flash read
    std::vector<uint32_t> readFlashPage(uint32_t) throw(ipDev::exception);
    std::vector<uint32_t> firmwareFromFlash(uint32_t, int) throw(ipDev::exception);

    //flash write
    void writeFlashPage(uint32_t, std::vector<rv>) throw(ipDev::exception);
    void eraseFlashSector(uint32_t) throw(ipDev::exception);
    void programFlash(const std::string&, uint32_t) throw(ipDev::exception);

    //flash verification
    void verifyFlash(const std::string&, uint32_t) throw(ipDev::exception);
    void printFlashPage(uint32_t) throw(ipDev::exception);

    //flash control
    void flashDoCommand(int) throw(ipDev::exception);
    void enableFlashWrite() throw(ipDev::exception);
    void waitForWriteDone() throw(ipDev::exception);
    void loadFlashT1() throw(ipDev::exception);
    void loadFlash() throw(ipDev::exception);

    //MCS file handling
    std::vector<uint32_t> firmwareFromMcs(const std::string&);

  private:
    //Other class objects used by AMC13_flash
    AMC13_utils* au;
    AMC13* amc13;

  };

}  

#endif //HCAL_AMC13_AMC13_FLASH_HH_INCLUDED
